import 'dart:html';
import 'package:intl/intl.dart';

void set(String key, String value, [bool permanent = false]) {
  _load();
  if (permanent == true || cookieMode == true)
    _localData[key] = value;
  else
    _sessionData[key] = value;
  _save();
}

void clear({bool clear_permanent: true}) {
  _sessionData.clear();
  if (clear_permanent == true || cookieMode == true) _localData.clear();
  _save();
}

String get(String key, [bool permanent = false]) {
  _load();
  if (permanent == true || cookieMode == true)
    return _localData.containsKey(key) ? _localData[key] : "";
  else
    return _sessionData.containsKey(key) ? _sessionData[key] : "";
}

bool contains(String key, [bool permanent = false]) {
  _load();
  return (permanent == true || cookieMode == true)
      ? _localData.containsKey(key)
      : _sessionData.containsKey(key);
}

void remove(String key, [bool permanent = false]) {
  _load();
  if (permanent == true || cookieMode == true)
    _localData.remove(key);
  else
    _sessionData.remove(key);
  _save();
}

bool get cookieMode => _cookieMode;
List<String> get localKeys => _localData.keys.toList(growable: false);
List<String> get sessionKeys => _sessionData.keys.toList(growable: false);
bool get loaded => _loaded;

void _load() {
  if (_loaded) return;
  try {
    if (window.sessionStorage == null)
      throw new Exception("sessionStorage disabled");
    if (window.localStorage == null)
      throw new Exception("localStorage disabled");

    window.sessionStorage
        .forEach((String key, String value) => _sessionData[key] = value);
    window.localStorage
        .forEach((String key, String value) => _localData[key] = value);

    /// this will trigger a DOMException 22 in iOS private mode, triggering cookies
    window.sessionStorage["test"] = "test";
    window.sessionStorage.remove("test");
    _cookieMode = false;
    _loaded = true;
  } catch (e) {
    String errMsg =
        "This page requires cookies to function properly, please enable it in your browser or contact us if the problem persists.";
    if (!window.navigator.cookieEnabled) {
      window.alert(errMsg);
      throw new Exception(errMsg);
    }
    // make sure cookies actually work
    DateTime timeNow = new DateTime.now().toUtc();
    DateTime localExpireTime = timeNow.add(new Duration(days: 365));
    DateFormat formatter = new DateFormat("E, dd LLL yyyy HH:mm:ss");
    window.document.cookie =
        "test=cookie;expires=" + formatter.format(localExpireTime) + " UTC;";
    if (!window.document.cookie.contains("test")) {
      window.alert(errMsg);
      throw new Exception(errMsg);
    }

    List<String> pairs = window.document.cookie.split("; ");
    pairs.forEach((String pair) {
      List<String> keyValue = pair.split("=");
      _localData[keyValue.first] = keyValue.last;
    });
    _cookieMode = true;
    _loaded = true;
  }
}

void _save() {
  if (!loaded) _load();
  if (_cookieMode == true) {
    // Delete all previous cookies
    List<String> pairs = window.document.cookie.split("; ");
    pairs.forEach((String pair) {
      List<String> pairs = pair.split("=");
      window.document.cookie =
          pairs.first + "=;expires=Thu, 01 Jan 1970 00:00:00 GMT;path=/";
    });

    /// Have the cookie to expire in two hours from now
    DateTime timeNow = new DateTime.now().toUtc();
    DateTime localExpireTime = timeNow.add(new Duration(days: 365));
    DateFormat formatter = new DateFormat("E, dd LLL yyyy HH:mm:ss");

    _localData.forEach((String key, String value) {
      window.document.cookie = key +
          "=" +
          value +
          ";expires=" +
          formatter.format(localExpireTime) +
          " UTC;path=/";
    });
  } else {
    window.sessionStorage.clear();
    _sessionData.forEach(
        (String key, String value) => window.sessionStorage[key] = value);
    window.localStorage.clear();
    _localData.forEach(
        (String key, String value) => window.localStorage[key] = value);
  }
}

bool _loaded = false;
bool _cookieMode = false;

Map<String, String> _sessionData = new Map();
Map<String, String> _localData = new Map();
